/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

/**
 * Abstraktní třída popisuje element, který může být součástí proudu
 * může to být trubka, konická trubka, izolovaná trubka, armatura, ventil
 * pumpa a podobně. V1.1
 * @author V. Hlubuček
 */
public abstract class Element {
    /**
     * Faktor 273,15 pro přepočet na a z absolutní teploty
     */
    public final double TABS = 273.15;
    /**
     * Název elementu (trubky a pod.)
     */
    protected String nazElementu;		// název elementu (trubky)
    /**
     * Vstupní a výstupní teploty a tlaky proudu
     */
    protected double tVstup, tVystup, pVstup, pVystup;	// vstupní a výstupní teploty a tlaky
    /**
     * Vstupní a výstupní rychlost na elementu
     */    
    protected double rychlVstup,rychlVystup;    // vstupní a výstupní rychlost    
    /**
     * Textové výsledky z elementu
     */
    protected String report;
    /**
     * Proud, jemuž element náleží
     */
    protected Proud pr;             // proud, jemuž element náleží
    protected String[] lTexty;
    protected String[] tValue;

    abstract public void dosadInteraktivniHodnoty();
/**
 * Interaktivní zadání parametrů elementu
 */    
    public void zadejParametryInteraktivne()  {
        JDialogElementInp jdei = new JDialogElementInp(null, true, lTexty, tValue); // interaktivní vstup dat elementu
        jdei.setVisible(true); 
        dosadInteraktivniHodnoty();        

    }    
/**
 * Pomocná funkce převádí string na hodnotu double
 * @param tx
 * @return 
 */    
    protected double doubleFromString(String tx)  {
        return (tx.equals("") || tx.equals("?") || tx.equals("*")) ? 0:Double.parseDouble(tx.replace(',', '.'));
    }
    
   /**
    * Výpočet tlakové ztráty na elementu
    * @param pr Proud
    * @return tlaková ztráta (Pa)
    */
    public abstract double tlakZtrata();	// tlaková ztráta na trubce (elementu) 
    /**
     * Předává výsledky za element do textové proměnné
     * @return výsledky String
     */
    public abstract String vysledkyElement();	// předání výsledků ve Stringu 

 /**
  * Vypočte koef. tření v trubce v závislosti na režimu
  * @param re  Reynoldsovo číslo
  * @param d   průměr trubky
  * @param eps abs.drsnost potrubí
  * @return koef. tření
  */   
    protected double hltr(double re, double d, double eps) {
        double hl, ht, reT;
        if (re < 2200) {
            return 64 / re;     // laminární
              }
        if (re < 4000) {        // přechodová oblast
          hl = 0.0291;
          reT = 4000;
          ht = hltu1(reT, d, eps);
          return  hl + (ht - hl) * (re - 2200) / 1800;
        }
        return hltu1(re, d, eps);  // turbulentní     
    }
    
 /**
  * Výpočet koef. tření v trubce pro turbulentní oblast
  * @param re Reynoldsovo číslo
  * @param d  průměr trubky
  * @param eps  abs.drsnost potrubí
  * @return koef. tření
  */   
    protected double hltu1(double re, double d, double eps) {
        boolean iterate = true;
        double h = 0.1, z, zi, f, fi;
        while (iterate) {
            z = Math.sqrt(h);
            f = 1 / z + 0.8686 * Math.log(2.51 / re / z + eps / 3.71 / d);
            fi = -1 / h * (2.18 / (2.51 / z + eps * re / 3.71 / d) + 1);
            zi = z - f / fi;
            if (zi <= 0) {
                zi = 0.0001; h = zi * zi; continue;
            }
            h = zi * zi;
            if (Math.abs((zi - z) / zi) > 0.00001) continue;
            iterate = false;         
        }       
        return h;
    }
/**
 * Vrací textový výsledkový report elementu
 * @return report
 */
    public String getReport() {
        return report;
    }
/**
 * Vrací proud jemuž element náleží
 * @return 
 */
    public Proud getPr() {
        return pr;
    }
/**
 * Nastavuje proud jemuž element náleží
 * @param pr 
 */
    public void setPr(Proud pr) {
        this.pr = pr;
    }

/**
 * Vrací textový název elementu
 * @return název
 */
    public String getNazElementu() {
        return nazElementu;
    }
/**
 * Nastavuje textový název elementu
 * @param nazElementu 
 */
    public void setNazElementu(String nazElementu) {
        this.nazElementu = nazElementu;
    }
/**
 * Vrací teplotu na vstupu do elementu (°C)
 * @return teplota
 */
    public double gettVstup() {
        return tVstup;
    }
/**
 * Nastavuj teplotu na vstupu do elementu (°C)
 * @param tVstup 
 */
    public void settVstup(double tVstup) {
        this.tVstup = tVstup;
    }
/**
 * Vrací teplotu na výstupu do elementu (°C)
 * @return teplota 
 */
    public double gettVystup() {
        return tVystup;
    }
/**
 * Nastavuje teplotu na výstupu do elementu (°C)
 * @param tVystup 
 */
    public void settVystup(double tVystup) {
        this.tVystup = tVystup;
    }
/**
 * Vrací tlak na vstupu do elementu (Pa)
 * @return tlak
 */
    public double getpVstup() {
        return pVstup;
    }
/**
 * Nastavuje tlak na vstupu do elementu (Pa)
 * @param pVstup 
 */
    public void setpVstup(double pVstup) {
        this.pVstup = pVstup;
    }
/**
 * Vrací tlak na ývstupu do elementu (Pa)
 * @return tlak
 */
    public double getpVystup() {
        return pVystup;
    }
/**
 * Nastavuje tlak na ývstupu do elementu (Pa)
 * @param pVystup 
 */
    public void setpVystup(double pVystup) {
        this.pVystup = pVystup;
    }
/**
 * Vrací rychlost na vstupu elementu (m/s)
 * @return rychlost
 */
    public double getRychlVstup() {
        return rychlVstup;
    }
/**
 * Nastavuje rychlost na vstupu elementu (m/s)
 * @param rychlVstup 
 */
    public void setRychlVstup(double rychlVstup) {
        this.rychlVstup = rychlVstup;
    }
/**
 * Vrací rychlost na výstupu elementu (m/s)
 * @return rychlost
 */
    public double getRychlVystup() {
        return rychlVystup;
    }
/**
 * Nastavuje Vrací rychlost na vstupu elementu (m/s)
 * @param rychlVystup 
 */
    public void setRychlVystup(double rychlVystup) {
        this.rychlVystup = rychlVystup;
    }
    
}
