/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import fydatlib.Smes;
import java.util.HashSet;
import java.util.List;

/**
 * Třída popisuje stechiometrii a kinetické parametry chemické reakce.
 * Pracuje s empirickou mocninovou kinetickou rovnicí r = k.cA^nA.cB^nB....<br>
 * Parametry kinetiky reakce:  ko - refer. hodnota rychl. konst., To - ref. abs. teplota,<br>
 * eAkt - aktivační energie reakce (J/mol), reakční řády složek.<br>
 * c jsou koncentrace (mol/m3), dH reakční entalpie (J/mol)<br>
 * kR0 a tR0 je referenční rovnovážná konstanta při teplotě tR0(K)
 * Rychlostní konstanta se počítá jako: k = ko.exp[eAkt(T-To)/R/T/To]<br>
 * Pokud se zadá referenční teplota To(K) <= 0, pak  k= ko.exp[-E/R/T] 
 * @author V.Hlubuček
 */
public class Reakce {

    protected static int pocetReakci;         // počet reakcí (instancí)
    protected static String report;           // výsledek ze statické metody reagSlozky
    protected String VERZE = "V1.3";
    /**
     * Typ rovnováhy: 1- koncentrace; 2-parc. tlak; 3-molární zlomek
     */
    protected int rTyp;               // typ rovnováhy: 1- koncentrace; 2-parc. tlak; 3-molární zlomek
    /**
     * Čísla reaktantů a produktů
     */
    protected int[]iSlozky;           // čísla reaktantů a produktů
    /**
     * Stechiometrické koeficienty složek (produkty kladně)
     */
    protected int[] iStech;           // stechiometrické koeficienty složek (produkty kladně)
    /**
     * Pole s čísli složek určujících kinetiku reakce
     */
    protected int[] iKinSloz;         // čísla "kinetických" složek
    protected String nazevReakce;     // textový popis reakce
    /**
     * Ref. rychl. konst, ref. abs. teplota
     */
    protected double k0, t0;          // ref. rychl. konst, ref. abs. teplota/**
    /**
     * Ref. rovnovážná konst, ref. abs. teplota
     */
    protected double kR0, tR0;        // ref. rovnovážná konst, ref. abs. teplota
    /**
     * Aktivační energie (J/mol), reakční entalpie (J/mol)
     */
    protected double eAkt, dH;        // aktivační energie (J/mol), reakční entalpie (J/mol)
    protected double[] radSlozky;     // reakční řády
    
    public Reakce() {
        pocetReakci++;
        nazevReakce = "Reakce " + pocetReakci;
    }
/**
 * Konstruktor třídy s prametry reakce
 * @param iSlozky   pole s čísly reakčních složek (int)
 * @param iStech    stechiometrické koeficienty reakčních složek (int); produkty kladně
 * @param k0    referenční hodnota rychlostní konstanty 
 * @param t0    referenční teplota (K); pokud je <= 0 je k0 předexponenciální faktor
 * @param eAkt  aktivační energie reakce (J/mol)
 * @param dH  reakční entalpie (J/mol) (záporně exotermní)
 * @param kR0 hodnota rovnovážné konstanty při referenční teplotě
 * @param tR0 hodnota referenční teploty rovnovážné konstanty (K)
 * @param rTyp typ chemické rovnováhy: 1- koncentrace; 2-parc. tlak; 3-molární zlomek 
 * @param iKinSloz  čísla složek na kterých závisí rychlost reakce
 * @param radSlozky reakční řády složek z iKinSloz[]
 */    
    public Reakce(int[] iSlozky, int[] iStech, double k0, double t0, double eAkt,
            double dH, double kR0, double tR0, int rTyp, int[] iKinSloz, double[] radSlozky) {
        pocetReakci++;
        nazevReakce = "Reakce " + pocetReakci;
        this.iSlozky = iSlozky;
        this.iStech = iStech;
        this.k0 = k0;
        this.t0 = t0;
        this.eAkt = eAkt;
        this.dH = dH;
        this.kR0 = kR0;
        this.tR0 = tR0;
        this.rTyp = rTyp;
        this.iKinSloz = iKinSloz;
        this.radSlozky = radSlozky;
    }
/**
 * Konstruktor třídy s prametry reakce (bez zadání rovnováhy)
 * @param iSlozky   pole s čísly reakčních složek (int)
 * @param iStech    stechiometrické koeficienty reakčních složek (int); produkty kladně
 * @param k0        referenční hodnota rychlostní konstanty
 * @param t0        referenční teplota (K); pokud je <= 0 je k0 předexponenciální faktor
 * @param eAkt      aktivační energie reakce (J/mol)
 * @param dH        reakční entalpie (J/mol) (záporně exotermní)
 * @param iKinSloz  čísla složek na kterých závisí rychlost reakce
 * @param radSlozky reakční řády složek z iKinSloz[] 
 */    
    public Reakce(int[] iSlozky, int[] iStech, double k0, double t0, double eAkt,
            double dH, int[] iKinSloz, double[] radSlozky) {
        pocetReakci++;
        nazevReakce = "Reakce " + pocetReakci;
        this.iSlozky = iSlozky;
        this.iStech = iStech;
        this.k0 = k0;
        this.t0 = t0;
        this.eAkt = eAkt;
        this.dH = dH;
        this.kR0 = 0;
        this.tR0 = 0;
        this.rTyp = 0;        
        this.iKinSloz = iKinSloz;
        this.radSlozky = radSlozky;
    }

    /**
 * Vyčíslí hodnotu rychlostní konstanty reakce pro zadanou abs. teplotu 
 * @param t absolutní teplota (K)
 * @return rychlostní konstanta
 */
    public double rychlKonst(double t) {
        double k;
        if (t0 > 0) {
            k = k0 * Math.exp(eAkt*(t - t0)/8.314/t/t0);
        } else {
            k = k0 * Math.exp(-eAkt/8.314/t);
        }
        return k;
    }
    
/**
 * Vyčíslí hodnotu rovnovážné konstanty reakce pro zadanou abs. teplotu 
 * @param t absolutní teplota (K)
 * @return rovnovážná konstanta
 */
    public double rovnKonst(double t) {
        double k;
        if (kR0 > 0) {
            k = kR0 * Math.exp(dH*(t - tR0)/8.314/t/tR0);
        } else {
            k = 0;
        }
        return k;
    }

/**
 * Interaktivní zadání parametrů reakce
 */    
    public void zadejParametryInteraktivne()  {
        JDialogReakceInp jdri = new JDialogReakceInp(null, true); // interaktivní vstup dat proudu
        jdri.setVisible(true); 
        Reakce rea = (jdri.getReakci());
        this.nazevReakce = rea.nazevReakce;
        this.iSlozky = rea.iSlozky;
        this.iStech = rea.iStech;
        this.iKinSloz = rea.iKinSloz;
        this.radSlozky = rea.radSlozky;
        this.k0 = rea.k0;
        this.t0 = rea.t0;
        this.eAkt = rea.eAkt;
        this.dH = rea.dH; 
    }
/**
 * Zjistí pole neopakujících se reagujících složek pro nanou směs a seznam reakcí
 * @param listReakci List<Reakce>
 * @param sm Smes
 * @return int[] reagujících složek
 */
    public static int[] reagSlozky(List<Reakce> listReakci, Smes sm) {
        // zjistí pole neopakujících se reagujících složek
        HashSet<Integer> reagSl = new HashSet();    // aby byl seznam jedinečných složek
        report = "";
        for (Reakce r : listReakci) {
            report += r.getNazevReakce() + String.format("%nk0= %6.2e  t0(K)= %4.1f  Eakt(kJ/mol)= %5.2f  dH(kJ/mol)= %6.2f%nKrovn(TR0)=%8.5g  TR0(K)= %4.1f  typRov= %1d%n", 
                    r.getK0(),r.getT0(),r.geteAkt()/1000,r.getdH()/1000,r.kR0, r.tR0, r.rTyp);
            for (int i = 0; i < r.getiKinSloz().length;i++) {
               report += String.format("Sl.kin. = %3d  řád= %4.2f%n", r.getiKinSloz()[i],r.getRadSlozky()[i]);
            }
            for (int j = 0; j < r.getiSlozky().length; j++) {
                int jj;
                jj = r.getiSlozky()[j];  
                if(jj != 0) {
                    reagSl.add(jj);  // přidají se jen dosud neexistující a nenulové prvky (Hash set)
//                    System.out.println("reasl: "+jj+"  j= "+j);
                    report += String.format("Sl.reak, stech : %4d  %2d  %s%n",jj, r.iStech[j], sm.getNazevSlozky(sm.porCisloLatky(Math.abs(jj)))); 
                }
            }
            report += String.format("(Kladně produkty, záporně reaktanty)%n");
        }
        int[] rs = new int[reagSl.size()];
        int j = 0;
        for (int ii: reagSl) {rs[j] = ii; j++; }
        return rs;
    }
/**
 * Vrací tisk výsledků z reagSlozky jako String
 * @return String
 */
    public static String vratVysledek() {
        return report;
    }
    
/**
 * Vrací true, je-li aspoň jedna z reakcí ze seznamu rovnovážnou
 * @return boolean
 */
    public static boolean jeRovn(List<Reakce> listReakci) {
        boolean rovn = false;
        for(Reakce r: listReakci) {
            if(r.rTyp > 0) rovn = true;
        }
        return rovn;
    }
    
    
/**
 * Vrací pole s čísly reakčních složek (int)
 * @return pole int
 */
    public int[] getiSlozky() {
        return iSlozky;
    }
/**
 * Nastavuje pole s čísly reakčních složek (int)
 * @param iSlozky 
 */
    public void setiSlozky(int[] iSlozky) {
        this.iSlozky = iSlozky;
    }
/**
 * Vrací stechiometrické koeficienty reakčních složek (int); produkty kladně
 * @return stech. koeficient (int)
 */
    public int[] getiStech() {
        return iStech;
    }
/**
 * Nastavuje stechiometrické koeficienty reakčních složek (int); produkty kladně
 * @param iStech 
 */
    public void setiStech(int[] iStech) {
        this.iStech = iStech;
    }
/**
 * Vrací čísla složek na kterých závisí rychlost reakce
 * @return 
 */
    public int[] getiKinSloz() {
        return iKinSloz;
    }
/**
 * Nastavuje čísla složek na kterých závisí rychlost reakce
 * @param iKinSloz celočíselné pole s čísly reakčních složek
 */
    public void setiKinSloz(int[] iKinSloz) {
        this.iKinSloz = iKinSloz;
    }
/**
 * Vrací textový popis reakce
 * @return String
 */
    public String getNazevReakce() {
        return nazevReakce;
    }
/**
 * Nastavuj textový popis reakce
 * @param nazevReakce textový popis reakce
 */
    public void setNazevReakce(String nazevReakce) {
        this.nazevReakce = nazevReakce;
    }
/**
 * Vrací referenční hodnotu rychlostní konstanty
 * @return 
 */
    public double getK0() {
        return k0;
    }
/**
 * Nastavuje referenční hodnotu rychlostní konstanty
 * @param k0 referenční hodnota konstanty při referenční teplotě
 */
    public void setK0(double k0) {
        this.k0 = k0;
    }
/**
 * Vrací referenční abs. teplotu (K)
 * @return teplota
 */
    public double getT0() {
        return t0;
    }
/**
 * Nastavuje referenční abs. teplotu (K)
 * @param t0 referenční abs. teplota 
 */
    public void setT0(double t0) {
        this.t0 = t0;
    }
/**
 * Vrací hodnotu aktivační energie (J/mol)
 * @return akt. energie
 */
    public double geteAkt() {
        return eAkt;
    }
/**
 * Nastavuje hodnotu aktivační energie (J/mol)
 * @param eAkt hodnota aktivační energie 
 */
    public void seteAkt(double eAkt) {
        this.eAkt = eAkt;
    }
/**
 * Vrací hodnotu reakční entalpie (J/mol)
 * @return reakční entalpie (J/mol)
 */
    public double getdH() {
        return dH;
    }
/**
 * Nastavuje hodnotu reakční entalpie (J/mol)
 * @param dH reakční entalpie
 */
    public void setdH(double dH) {
        this.dH = dH;
    }
/**
 * Vrací hodnotu rovnovážné konstanty
 * @return konstanta double
 */
    public double getkR0() {
        return kR0;
    }
/**
 * Nastavuje hodnotu rovnovážné konstanty
 * @param kR0 
 */
    public void setkR0(double kR0) {
        this.kR0 = kR0;
    }
/**
 * Vrací ref. teplotu rovnovážné konstanty (K)
 * @return teplota double
 */
    public double gettR0() {
        return tR0;
    }
/**
 * Nastavuje ref. teplotu rovnovážné konstanty (K)
 * @param tR0 
 */
    public void settR0(double tR0) {
        this.tR0 = tR0;
    }
    
/**
 * Vrací pole s reakčními řády
 * @return řády (double)
 */
    public double[] getRadSlozky() {
        return radSlozky;
    }
/**
 * Nastavuje pole s reakčními řády
 * @param radSlozky kinetický řád složky v mocninové kin. rovnici
 */
    public void setRadSlozky(double[] radSlozky) {
        this.radSlozky = radSlozky;
    }
/**
 * Vrací typ chemické rovnováhy
 * 1- koncentrace; 2-parc. tlak; 3-molární zlomek
 * @return int rTyp 
 */
    public int getrTyp() {
        return rTyp;
    }
/**
 * Nastavuje typ chemické rovnováhy
 * 1- koncentrace; 2-parc. tlak; 3-molární zlomek
 * @param rTyp 
 */
    public void setrTyp(int rTyp) {
        this.rTyp = rTyp;
    }
    
/**
 * Vrací verzi třídy
 * @return String
 */
    public String getVERZE() {
        return VERZE;
    }
    
    
}
